/*
Copyright (c) 2003 Forward Computing and Control Pty. Ltd.
NSW. Australia,  www.forward.com.au
All rights reserved.

Redistribution of the source of this package, with or without
modification, is NOT permitted.

Redistribution of the compiled java .class files of this package, with or without
modification, are permitted provided that the following conditions
are met:
1. Redistributions in compiled java .class form must reproduce the above
   copyright notice, this list of conditions and the following disclaimer
   in the documentation and/or other materials provided with the distribution.

2. If the software is modified, the compiled java .class distribution must
   include, in the documentation and/or other materials provided with the 
   distribution, a statement that the software has been modified and
   identify the person who was responsable for the modifications.  

3. Neither the name of the author nor Forward Computing and Control may be 
   used to endorse or promote products derived from this software
   without specific prior written permission.

4.  All advertising materials mentioning features or use of this
    software must display the following acknowledgment: 
    "This product includes software developed by Matthew Ford
     and Forward Computing and Control Pty. Ltd. 
    (http://www.forward.com.au)."


THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*/
package au.com.forward.test;

import java.awt.*;
import java.awt.event.*;
import java.util.Date;
import java.util.Vector;

import javax.swing.*;
import javax.swing.plaf.basic.BasicComboBoxEditor;
import javax.swing.text.*;
import au.com.forward.inputTipsManager.*;

/**
 *  Description of the Class
 *
 *@author     Matthew Ford
 * created    October 16, 2004
 */
public class InputTipsEditorTestPanel extends JPanel {

	JPanel contentPane;

	JPanel jPanel1 = new JPanel();

	FlowLayout flowLayout1 = new FlowLayout();

	GridLayout gridLayout1 = new GridLayout();

	Vector data = new Vector();


	/**
	 *  Constructor for the InputTipsEditorTestPanel object
	 */
	public InputTipsEditorTestPanel() {
		contentPane = this;
		contentPane.setLayout(flowLayout1);
		jPanel1.setLayout(gridLayout1);
		gridLayout1.setRows(8);
		gridLayout1.setColumns(2);
		gridLayout1.setHgap(20);

		JTextField keyText = new JTextField();
		JTextField focusText = new JTextField();
		JTextField inputText = new JTextField();
		JTextField modelText = new JTextField();

		// now create the JComboBoxes
		data.add("123");
		data.add("456");
		data.add("a23");// one bad
		JComboBox keyCombo = new JComboBox(data);
		keyCombo.setEditable(true);
		JComboBox focusCombo = new JComboBox(data);
		//focusCombo.setEditable(true);
		JComboBox inputCombo = new JComboBox(data);
		//inputCombo.setEditable(true);
		JComboBox modelCombo = new JComboBox(data);
		modelCombo.setEditable(true);

		keyText.addKeyListener(new KeystrokeListener());

		// for JComboBox need to get underlying JTextField
		((JTextField) ((BasicComboBoxEditor) keyCombo.getEditor()).getEditorComponent()).addKeyListener(new KeystrokeListener());

		focusText.addFocusListener(new TextFocusListener());
		if (focusCombo.isEditable()) {
			// for JComboBox need to get underlying Editor
			((JTextField) ((BasicComboBoxEditor) focusCombo.getEditor()).getEditorComponent()).addFocusListener(new TextFocusListener());
		} else {
			focusCombo.addFocusListener(new TextFocusListener());
		}

		inputText.setInputVerifier(new TextInputVerifier());
		if (inputCombo.isEditable()) {
			((JTextField) ((BasicComboBoxEditor) inputCombo.getEditor()).getEditorComponent()).setInputVerifier(new TextInputVerifier());
		} else {
			inputCombo.setInputVerifier(new TextInputVerifier());
		}

		modelText.setDocument(new IntegerDocument());
		((JTextField) ((BasicComboBoxEditor) modelCombo.getEditor()).getEditorComponent()).setDocument(new IntegerDocument());

		contentPane.add(jPanel1);
		jPanel1.add(new JLabel("Key Listener"));
		jPanel1.add(keyText);
		jPanel1.add(new JLabel("Focus Listener"));
		jPanel1.add(focusText);
		jPanel1.add(new JLabel("Input Verifier"));
		jPanel1.add(inputText);
		jPanel1.add(new JLabel("Model"));
		jPanel1.add(modelText);
		jPanel1.add(new JLabel("Key Combo Listener"));
		jPanel1.add(keyCombo);
		jPanel1.add(new JLabel("Focus Combo Listener"));
		jPanel1.add(focusCombo);
		jPanel1.add(new JLabel("Input Combo Listener"));
		jPanel1.add(inputCombo);
		jPanel1.add(new JLabel("Model Combo Listener"));
		jPanel1.add(modelCombo);
	}




	/**
	 *  Description of the Class
	 *
	 *@author     Matthew Ford
	 *@created    October 16, 2004
	 */
	class KeystrokeListener extends KeyAdapter {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void keyTyped(KeyEvent e) {
			char c = e.getKeyChar();
			if (!(Character.isDigit(c)
					 || (c == KeyEvent.VK_BACK_SPACE)
					 || (c == KeyEvent.VK_DELETE)
					 || (c == '\u0016')
			/*
			 *  generated by Ctrl+V when pasting
			 */
					 || (c == KeyEvent.VK_ESCAPE)
					 || (c == KeyEvent.VK_ENTER)
					)) {
				InputTipsManager.sharedInstance()
						.showInputTip((JComponent) e.getSource(),
						"<HTML>" + InputTipsManager.getKeyText(e) 
						 + " not valid here<BR> Only numbers are allowed.");

				e.consume();
			} else {
				InputTipsManager.sharedInstance().hideInputTip();
			}
		}
	}


	/**
	 *  Description of the Class
	 *
	 *@author     Matthew Ford
	 *@created    October 16, 2004
	 */
	class TextFocusListener extends FocusAdapter {
		/**
		 *  Description of the Method
		 *
		 *@param  e  Description of the Parameter
		 */
		public void focusLost(FocusEvent e) {
			String content = "";
			JComponent comp = (JComponent) e.getSource();
			if (comp instanceof JTextField) {
				JTextField textField = (JTextField) comp;
				content = textField.getText();
			} else if (comp instanceof JComboBox) {
				JComboBox comboBox = (JComboBox) comp;
				content = "" + comboBox.getSelectedItem();
			}
			if (content.length() != 0) {
				try {
					Integer.parseInt(content);
				} catch (NumberFormatException nfe) {
					comp.requestFocus();
					InputTipsManager.sharedInstance().showInputTip(
							comp, "Invalid number '" + content + "'");
					return;
				}
			}
			InputTipsManager.sharedInstance().hideInputTip();
		}
	}


	/**
	 *  Description of the Class
	 *
	 *@author     Matthew Ford
	 *@created    October 16, 2004
	 */
	class TextInputVerifier extends InputVerifier {
		/**
		 *  Description of the Method
		 *
		 *@param  comp  Description of the Parameter
		 *@return       Description of the Return Value
		 */
		public boolean verify(JComponent comp) {
			boolean returnValue = true;
			String content = "";
			if (comp instanceof JTextField) {
				JTextField textField = (JTextField) comp;
				content = textField.getText();
			} else if (comp instanceof JComboBox) {
				JComboBox comboBox = (JComboBox) comp;
				content = "" + comboBox.getSelectedItem();
			}
			if (content.length() != 0) {
				try {
					Integer.parseInt(content);
				} catch (NumberFormatException e) {
					InputTipsManager.sharedInstance().showInputTip(
							comp, "Invalid number '" + content + "'");
					returnValue = false;
				}
			}
			InputTipsManager.sharedInstance().hideInputTip();
			return returnValue;
		}
	}


	/**
	 *  Description of the Class
	 *
	 *@author     Matthew Ford
	 *@created    October 16, 2004
	 */
	static class IntegerDocument extends PlainDocument {

		/**
		 *  Description of the Method
		 *
		 *@param  offset                    Description of the Parameter
		 *@param  string                    Description of the Parameter
		 *@param  attributes                Description of the Parameter
		 *@exception  BadLocationException  Description of the Exception
		 */
		public void insertString(int offset, String string,
				AttributeSet attributes) throws BadLocationException {

			if (string == null) {
				return;
			}
			// else
			String newValue;
			int length = getLength();
			if (length == 0) {
				newValue = string;
			} else {
				String currentContent = getText(0, length);
				StringBuffer currentBuffer = new StringBuffer(currentContent);
				currentBuffer.insert(offset, string);
				newValue = currentBuffer.toString();
			}
			try {
				Integer.parseInt(newValue);
				super.insertString(offset, string, attributes);
				InputTipsManager.sharedInstance().hideInputTip();
			} catch (NumberFormatException exception) {
				InputTipsManager.sharedInstance().showInputTip(
						"<HTML>Invalid input '<FONT COLOR=\"RED\">" + string + "</FONT>'<BR> Only numbers allowed.");
			}
		}


		/**
		 *  Description of the Method
		 *
		 *@param  offset                    Description of the Parameter
		 *@param  len                       Description of the Parameter
		 *@exception  BadLocationException  Description of the Exception
		 */
		public void remove(int offset, int len) throws BadLocationException {
			super.remove(offset, len);
			int length = getLength();
			if (length == 0) {
				InputTipsManager.sharedInstance().hideInputTip();
				return;
			}
			String currentContent = getText(0, length);
			try {
				Integer.parseInt(currentContent);
				InputTipsManager.sharedInstance().hideInputTip();
			} catch (NumberFormatException exception) {
				InputTipsManager.sharedInstance().showInputTip(
						"Current input is invalid. Only numbers allowed.");
			}
		}

	}
}

